;***************************************************************************;
;    Nixie.asm        Ronald Dekker   spring 2005                           ;
;                     program for nixie (promotion) clock                   ;
;***************************************************************************;
;
;na een (hardware) reset komt de klok eerste in de "mode select toestand"
;hierbij telt het uiterst rechte buisje cyclisch van 0...6. Door de knop
;tijdens een van de getallen in te drukken wordt een van de modi gekozen.
;mode 0: standaard klok. na het indrukken van de toets moet de klok eerst 
;        gelijk gezet worden. automatisch gaan hiertoe de uren "lopen"
;        door de toets nogmaals in te drukken stoppen de uren op een bepaalde
;        tijd en gaan de minuten "lopen". Na weer de toets in ge drukt te 
;        hebben blijven ook de minuten staan op de gekozen tijd. De seconden
;        staan op 00. Het nogmaals indrukken van de toets start de klok.
;        wordt de toets ingedrukt als de klok gewoon loopt wordt een 
;        eventuele blanking opgeheven (zie mode 6) en komt de klok weer in de
;        "mode select toestand"
;mode 1: in deze toestand kunnen de buisjes getest worden. De start toestand
;        is 00:00:00 na het indrukken van de toets wordt dit 11:11:11 etc.
;        als de toets nogmaals wordt ingedrukt op 99:99:99 keert de klok 
;        weer terug in de "mode select toestand".
;mode 2: net als in mode 1, maar nu wordt het display automatisch iedere
;        seconde verhoogd en gaat dit oneindig door. De enige manier om uit
;        deze toetstand te raken is een harware reset
;mode 3: in deze test mode worden er eerst nullen naar p0,p1,p2 geschreven.
;        (alle buisjes gedoofd). Na het indrukken van de knop worden er alleen
;        maar enen naar de poorten geschreven (alleen het meest linkse buisje
;        toont 0 en 1 tegelijk). Na het weer indrukken van de knop weer
;        allemaal nullen etc. Na 5 van deze cycli komt de klok weer in de 
;        "mode select toestand".  
;mode 4: in deze test toestand worden de poorten snel geteld. bit 0 van poort
;        p0 het snelst, bit 7 van poort2 het langzaamst. Ook deze mode kan 
;        alleen dmv een hardware reset verlaten worden.
;mode 5: deze mode set de "leading zero supress flag" hierdoor wordt bijv.
;        i.p.v. 5:03:06 nu 5: 3: 6 weergegeven. De flag kan alleen gereset 
;        worden d.m.v. een hardware reset. Notabene, de meest linkse nul
;        wordt nooit weergegeven. Direct na het indrukken van de knop komt
;        de klok weer in de "mode select" toestand.
;mode 6: deze mode set de "night savings flag". Hierdoor worden de buisjes
;        uitgezet tussen 00:00:00 en 06:00:00 s'nachts. Dit spaart de buisjes.
;        deze flag wordt weer gereset door de knop in te drukken als de 
;        buisjes uit zijn ( de klok raakt dan tevens weer in de "mode select
;        toestand") of d.m.v. een hardware reset.
;
; Assembler directives:
;
NOFLD
WIDTH 132
PAGE 255
;
; Internal memory orginization:
; 00-1F :scratch pad
; 20-2F :bit's
; 50-6F :stack
;
;
; The constants:
stackp		equ	050H		;location of the stack pointer
T1_cyc		equ	028H		;40 cycles of 250us = 10ms
bnc_rst		equ	005H		;number oof display cycles for debounce
;
;
; Bytes locations:
indexr		equ	000H		;index register R0 in bank 0
;
T1_cnt		equ	001H
sec		equ	002H		;seconds
min		equ	003H		;minutes
hrs		equ	004H		;hours
p0_e		equ	005H		;output byte p0_even (min)
p1_e		equ	006H		;output byte p1_even (hrs)
p2_e		equ	007H		;output byte p2_even (sec)
p0_o		equ	008H		;output byte p0_odd  (min)
p1_o		equ	009H		;output byte p1_odd  (hrs)
p2_o		equ	00AH		;output byte p2_odd  (sec)
bnc_cnt		equ	00BH		;bounce counter for input key
cnt1		equ	00CH
cnt2		equ	00DH
mode		equ	00EH		;mode byte
;
; Bit locations:
o_e		equ	000H		;current display 0=even, 1=odd
sec_inp		equ	001H		;save bit for current value of p3.4
sec_old		equ	002H		;previous value of p3.4
sec_new		equ	003H		;set if a new second is detected
key		equ	004H		;set if the key is pressed
lz_sup		equ	005H		;set if leading zeros need suppresed
blank		equ	006H		;set if in blank mode
tbs_off		equ	007H		;set if tubes are actually blanked
;
;
		org	0000H
		ljmp	main
;
; T1 interrupt service routine
;
		org	001BH
		lcall	T1_int		;call timer 1 service routine
		reti
;
;
; main program		
;
		org	0100H
main:		mov	SP,#stackp	;initialize stack-pointer
		mov	sec,#000H	;set time to 12:00:00
		mov	min,#000H	;
		mov	hrs,#000H	;
		clr	key		;no key pressed
		clr	lz_sup		;suppress leading zeros
		lcall	init_tmr	;initialize and start Timer 1
;
main1:		lcall	getmode		;select mode
					;
		cjne	A,#000H,main4   ;mode=0 (run time)
		lcall	settime		;	
main2:		lcall	getsec		;poll seconds input
		jnb	sec_new,main3	;if no sec. look for key
		clr	sec_new		;if sec inc time
		mov	IE,#008H	;
		lcall	inctime		;
		lcall	chk_blnk	;
		lcall	convert		;
		mov	IE,#088H	;
main3:		jnb	key,main2	;if key goto select mode
		clr	key		;
		sjmp	main1		;	
					;
main4:		cjne	A,#001H,main5   ;mode=1 (inc. digits on key)
		lcall	test_A		;
		sjmp	main1		;
					;
main5:		cjne	A,#002H,main6   ;mode=2 (inc. digits on sec, inf)
		lcall	test_C		;
		sjmp	main1		;pro-forma jump (we never execute it)
					;
main6:		cjne	A,#003H,main7	;mode=3 (alternate zero's/one's to 
		lcall	test_B		;to outputs
		sjmp	main1		;
					;
main7:		cjne	A,#004H,main8	;mode=4 (fast counter on output, inf)
		lcall	test_D		;
		sjmp	main1		;
					;
main8:		cjne	A,#005H,main9   ;mode=5 (leading zero suppres)
		setb	lz_sup		;
		sjmp	main1		;
					;
main9:		cjne	A,#006H,main1   ;mode=6 (night savings on)
		setb	blank		;
		sjmp	main1		;
;
;******************************************************************************
; routine T1_int 
;******************************************************************************
;This routine is the timer 1 sevice routine. It is called by an interrupt 
;generated by a timer 1 overflow every 250 us. The routine takes care of 
;multiplexing the nixie tubes and of reading the input button. Since the nixie
;tubes need changing only about every 10ms, counter T1_cnt is used to count 
;down T1_cyc cycles to reach this delay. Every display cycle either the odd
;digits or the odd digits of the tubes are actived. 
;
;inputs:   T1-cyc  I : constant with reload value foor T1_cnt 
;          T1_cnt  I : counter used to count down the number of T1 interupts
;	   p0_o	   I : display data for port 0 during an odd display cycle
;	   p1_o	   I : display data for port 1 during an odd display cycle
;	   p2_o	   I : display data for port 2 during an odd display cycle
;	   p0_e	   I : display data for port 0 during an even display cycle
;	   p1_e	   I : display data for port 1 during an even display cycle
;	   p2_e	   I : display data for port 2 during an even display cycle
;          o_e	   B : 1 if current display cycle is odd, 0 if even
;outputs: 
;internal: 
;registers used: P0,P1,P2 
;subroutines used: 
;
; push A,BPSW,DPTR,indexr
; T1_cnt = T1_cnt - 1
; if T1_cnt = 0 then
;      T1_cnt = T1_cyc
;      first deal with multiplexing of the displays
;      if o_e = 0 then
;         o_e = 1
;         write odd set to ports
;      else
;         o_e = 0
;         write even set to ports
;      endif
;      next deal with the input key
;      if p3.2 = 0 then
;         if bnc_cnt <> 0 then
;            bnc_cnt = bnc_cnt - 1
;            if bnc_cnt = 0 then
;               key = 1
;            end if
;         endif
;      else
;         bnc_cnt = bnc_rst
;      endif
; endif

T1_int:		push	acc		;save working registers
		push	B		;
		push	PSW		;
		push	DPL		;
		push	DPH		;
		push	indexr		;
		djnz	T1_cnt,T1_int4	;decrement T1_cnt until 0
		mov	T1_cnt,#T1_cyc	;reload T1_cnt
		jb	o_e,T1_int1	;if o_e set then go display even
		setb	o_e		;if not go and display odd:
		mov	p0,p0_o		;reset o_e and write odd data
		mov	p1,p1_o		;to output ports
		mov	p2,p2_o		;
		sjmp	T1_int2		;	
T1_int1:	clr	o_e		;display even:
		mov	p0,p0_e		;clear o_e and write even data
		mov	p1,p1_e		;to output ports
		mov	p2,p2_e		;
T1_int2:	jb	p3.2,T1_int3	;continue if key pressed
		mov	A,bnc_cnt	;if bnc_cnt already 0 then exit
		jz	T1_int4		;
		djnz	bnc_cnt,T1_int4	;decrement bnc_cnt exit if 0
		setb	key		;key pressed
		sjmp	T1_int4		;exit
T1_int3:	mov	bnc_cnt,#bnc_rst;reload bounce counter		
T1_int4:	pop	indexr		;reload working registers
		pop	DPH		;
		pop	DPL		;
		pop	PSW		;
		pop	B		;
		pop	acc		;
		ret			;return
;
;******************************************************************************
; routine init_tmr 
;******************************************************************************
;This routine initializes timer T1 to mode 2 (8 bits with automatic reload).
;After a call to this routine, the timer will cause a highest level interrupt
;every 250 us. 
;
;inputs:   T1-cyc  I : constant with reload value foor T1_cnt 
;          T1_cnt  I : counter used to count down the number of T1 interupts
;outputs: 
;internal: 
;registers used: TMOD,TH1,TL1,IP,IE,TR1 
;subroutines used: 
;
init_tmr:	mov	T1_cnt,#T1_cyc  ;initialise the number of T1 cycles
		mov	TMOD,#020H	;T1 mode 2, T0 off
		mov	TH1,#256-250    ;initialiseer op 250 periodes
		mov	TL1,#256-250    ;
		mov 	IP,#08H		;T1 int. heeft hoogste priorietijd
		mov	IE,#088H	;enable T1 interrupts
		setb	TR1		;T1 aan
		ret
;
;******************************************************************************
; routine bcd2nix 
;******************************************************************************
;This routine convert the bcd code from the internal clock to the code required
;for driving the nixie tubes. This is needed for two reasons: 1.The nixie tubes
;are multiplexed to that half the time the odd and the oother half time the 
;even digits are shown. 2.The outputs of the 74141 drivers are somewhat
;arbitrarily connected to the nixie kathodes (it was just easyer this way out
;in the wiring). The following table shows how the bcd code is translated
;
;                bcd code |  0   1   2   3   4   5   6   7   8   9
;                         ----------------------------------------
; code during even cycle  |  6   7   4   7   5   7   1   7   0   7
; code during odd  cycle  |  7   6   7   4   7   5   7   1   7   0
;
;so when the bcd code is e.g. 3 a 7 (note 7 is not connected to a kathode so 
;this means tube off) is applied to the tube during an even cycle while a 4
;is connected during an odd cycle.
;
; A = A.and.0F
; A = A*2
; B = A
; A = @DPTR+A
; B <> A
; A = A + 1
; A = @DPTR+A
; B <> A
;
;inputs:   A	   I : The bcd digit to be translated (only lower nibble used) 
;outputs:  A       I : the code for the even cycle
;          B       I : the code for the odd cycle
;internal:
;registers used: DPTR 
;subroutines used: 
;
bcd2nix:	mov	DPTR,#nixtab	;point DPTR to translation table
		anl	A,#00FH		;label lowest nibble
		rl 	A		;multiply acc by 2
		mov	B,A		;save table offset in B
		movc	A,@A+DPTR	;get code for even cycle
		xch	A,B		;exhange A and B
		inc	A		;add 1 to A
		movc	A,@A+DPTR	;get code for odd cycle
		xch	A,B		;even in A and odd in B
		ret
;
nixtab:		db	0F6H,0F7H,0F7H,0F6H,0F4H,0F7H,0F7H,0F4H,0F5H,0F7H
		db	0F7H,0F5H,0F1H,0F7H,0F7H,0F1H,0F0H,0F7H,0F7H,0F0H
;
;******************************************************************************
; routine hrs2nix 
;******************************************************************************
;This routine converts the bcd code from the highest nibble of the hours from
;the internal clock to the code required for driving the nixie tubes. 
;This is needed for two reasons: 1.The nixie tubes
;are multiplexed to that half the time the odd and the oother half time the 
;even digits are shown. 2.The outputs of the 74141 drivers are somewhat
;arbitrarily connected to the nixie kathodes (it was just easyer this way out
;in the wiring). The reason that this routine differs from bcd2nix is that 
;for this tube the wires are somewhat differently connected. Besides this
;the table includes blanking of the trailing zero.
;The following table shows how the bcd code is translated.
;
;                bcd code |  0   1   2   3   4   5   6   7   8   9
;                         ----------------------------------------
; code during even cycle  |  1   1   2   1   1   1   1   1   1   1
; code during odd  cycle  |  1   3   1   1   1   1   1   1   1   1
;
;so when the bcd code is e.g. 1 a 1 (note 1 is not connected to a kathode so 
;this means tube off) is applied to the tube during an even cycle while a 3
;is connected during an odd cycle.
;
;since the higher 2 bits in the nible drive the even/odd anode drivers these
;bits are set in the table at the same time so that the table becomes:
;(bit 6 = odd, bit 7 = even)
;
;                bcd code |  0   1   2   3   4   5   6   7   8   9
;                         ----------------------------------------
; code during even cycle  |  9   9   A   9   9   9   9   9   9   9
; code during odd  cycle  |  5   7   5   5   5   5   5   5   5   5
;
;although normally during clock operation only 0,1,2 are used all the other
;bcd digits are also translated (to off) to facilitate the test mode.
;
; A = A.and.0F
; A = A*2
; B = A
; A = @DPTR+A
; B <> A
; A = A + 1
; A = @DPTR+A
; B <> A
;
;inputs:   A	   I : The bcd digit to be translated (only lower nibble used) 
;outputs:  A       I : the code for the even cycle
;          B       I : the code for the odd cycle
;internal:
;registers used: DPTR 
;subroutines used: 
;
hrs2nix:	mov	DPTR,#hrstab	;point DPTR to translation table
		anl	A,#00FH		;label lowest nibble
		rl 	A		;multiply acc by 2
		mov	B,A		;save table offset in B
		movc	A,@A+DPTR	;get code for even cycle
		xch	A,B		;exhange A and B
		inc	A		;add 1 to A
		movc	A,@A+DPTR	;get code for odd cycle
		xch	A,B		;even in A and odd in B
		ret
;
hrstab:		db	0F9H,0F5H,0F9H,0F7H,0FAH,0F5H,0F9H,0F5H,0F9H,0F5H
		db	0F9H,0F5H,0F9H,0F5H,0F9H,0F5H,0F9H,0F5H,0F9H,0F5H
;
;******************************************************************************
; convert 
;******************************************************************************
;This routine translates the internal time (in hrs,min and sec) to the correct
;codes for the p0_e,....p2_o bytes. 
;
;
;inputs:   hrs     I : hours byte of the internal time 
;          min     I : minutes byte of the internal time
;	   sec     I : seconds byte of the internal time
;	   lz_sup  B : set if leading zeros need supressing
;	   tbs_off B : set if tubes need to be switched off because of blanking
;outputs:  p0_o	   I : display data for port 0 during an odd display cycle
;	   p1_o	   I : display data for port 1 during an odd display cycle
;	   p2_o	   I : display data for port 2 during an odd display cycle
;	   p0_e	   I : display data for port 0 during an even display cycle
;	   p1_e	   I : display data for port 1 during an even display cycle
;	   p2_e	   I : display data for port 2 during an even display cycle
;internal: 
;registers used: P0,P1,P2 
;subroutines used: 
;
convert:	jnb	tbs_off,convert1;
		mov	p0_o,#077H	;switch off the tubes including	
		mov	p1_o,#017H	;anodes
		mov	p2_o,#077H	;
		mov	p0_e,#077H	;
		mov	p1_e,#017H	;
		mov	p2_e,#077H	;
		clr	tbs_off		;reset tbs_off bit
		sjmp	convert4	;exit
					;
convert1:	mov	p0_o,#077H	;off code to all output byte	
		mov	p1_o,#077H	;
		mov	p2_o,#077H	;
		mov	p0_e,#077H	;
		mov	p1_e,#0B7H	;
		mov	p2_e,#077H	;
;
		mov	A,sec		;convert lower nibble of seconds
		lcall	bcd2nix		;
		anl	p2_e,A		;lower nibble of sec even output byte
		mov	A,B		;get odd part
		anl	p2_o,A		;lower nibble of sec odd output byte
;
		mov	A,sec		;convert higher nibble of seconds
		swap	A		;
		lcall	bcd2nix		;
		jnb	lz_sup,convert2 ;see if leading 0 need suppressing
		cjne	A,#0F6H,convert2;if code for 0 (F6), blank digit by
		mov	A,#0F7H		;replacing it with F7
convert2:	swap	A		;higher nibble of sec even output byte
		anl	p2_e,A		;
		mov	A,B		;get odd part
		swap	A		;higher nibble of sec odd output byte
		anl	p2_o,A		;
;
		mov	A,min		;convert lower nibble of minutes
		lcall	bcd2nix		;
		anl	p0_e,A		;lower nibble of min even output byte
		mov	A,B		;get odd part
		anl	p0_o,A		;lower nibble of min odd output byte
;
		mov	A,min		;convert higher nibble of minutes
		swap	A		;
		lcall	bcd2nix		;
		jnb	lz_sup,convert3 ;see if leading 0 need suppressing
		cjne	A,#0F6H,convert3;if code for 0 (F6), blank digit by
		mov	A,#0F7H		;replacing it with F7
convert3:	swap	A		;higher nibble of min even output byte
		anl	p0_e,A		;
		mov	A,B		;get odd part
		swap	A		;higher nibble of min odd output byte
		anl	p0_o,A		;
;
		mov	A,hrs		;convert lower nibble of hours
		lcall	bcd2nix		;
		anl	p1_e,A		;lower nibble of hrs even output byte
		mov	A,B		;get odd part
		anl	p1_o,A		;lower nibble of hrs odd output byte
;
		mov	A,hrs		;convert higher nibble of hours
		swap	A		;
		lcall	hrs2nix		;
		swap	A		;higher nibble of hrs even output byte
		anl	p1_e,A		;
		mov	A,B		;get odd part
		swap	A		;higher nibble of hrs odd output byte
		anl	p1_o,A		;
convert4:	ret
;
;******************************************************************************
; convertm 
;******************************************************************************
;This routine translates the mode byte and writes it to the lowest nibble of
;the seconds
;
;inputs:   mode    I : clock mode 
;outputs:  p0_o	   I : display data for port 0 during an odd display cycle
;	   p1_o	   I : display data for port 1 during an odd display cycle
;	   p2_o	   I : display data for port 2 during an odd display cycle
;	   p0_e	   I : display data for port 0 during an even display cycle
;	   p1_e	   I : display data for port 1 during an even display cycle
;	   p2_e	   I : display data for port 2 during an even display cycle
;internal: 
;registers used: P0,P1,P2 
;subroutines used: 
;
convertm:	mov	p0_o,#077H	;off code to all output byte	
		mov	p1_o,#057H	;
		mov	p2_o,#077H	;
		mov	p0_e,#077H	;
		mov	p1_e,#097H	;
		mov	p2_e,#077H	;
;
		mov	A,mode		;convert lower nibble of seconds
		lcall	bcd2nix		;
		anl	p2_e,A		;lower nibble of sec even output byte
		mov	A,B		;get odd part
		anl	p2_o,A		;lower nibble of sec odd output byte
		ret
;
;******************************************************************************
; routine getsec 
;******************************************************************************
;This routine polls the seconds input pin (p3.4) and searches for an up going
;flank. When it has found such a flank, it sets the sec_new bit. This bit
;needs to be reset by the receiving routine.
;
;inputs:   sec_old B : status of p3.4 pin during last call to getsec 
;outputs:  sec_new B : set if an 0->1 transistion is found on pin p3.4
;internal: sec_inp B : value of p3.4 at the beginning of getsec
;registers used: C, p3.4
;subroutines used: 
;
; sec_inp = p3.4
; if sec_inp = 1 then
;    if sec_old = 0 then
;       sec_new = 1
;    endif
; endif
; sec_old = sec_inp
;
getsec:		mov	C,p3.4		;sec_inp=p3.4
		mov	sec_inp,C	;
		jnb	sec_inp,getsec1	;if input=0 return
		jb	sec_old,getsec1	;if previous bit=1 return
		setb	sec_new		;new second 
getsec1:	mov	C,sec_inp	;sec_old=sec_inp
		mov	sec_old,C
		ret
;
;******************************************************************************
; routine inctime 
;******************************************************************************
;This routine increments the time by one second and adjusts the minutes and
;hours accordingly.
;
;inputs:   hrs     I : hours byte of the internal time 
;          min     I : minutes byte of the internal time
;	   sec     I : seconds byte of the internal time
;outputs:  hrs     I : hours byte of the internal time 
;          min     I : minutes byte of the internal time
;	   sec     I : seconds byte of the internal time
;internal: 
;registers used: C
;subroutines used: 
;
;    sec = sec + 1
;    if sec = 60 then
;	sec = 0
;       min = min + 1
;       if min = 60 then
;          min = 0
;          hrs = hrs + 1
;          if hrs = 24 then
;             hrs = 0
;          endif
;       endif
;     endif
;
inctime:	clr	C		;
		mov	A,sec		;sec = sec + 1
		add	A,#001H		;
		da	A		;
		mov	sec,A		;
		cjne	A,#060H,inctime1 ;if not 60 then return else cont
		mov	sec,#000H	;sec = 0
		clr	C		;
		mov	A,min		;min = min + 1
		add	A,#001H		;
		da	A		;
		mov	min,A		;
		cjne	A,#060H,inctime1 ;if not 60 return else cont
		mov	min,#000H	;min = 0
		clr	C		;
		mov	A,hrs		;hrs = hrs + 1
		add	A,#001H		;
		da	A		;
		mov	hrs,A		;
		cjne	A,#024H,inctime1 ;if not 24 return else cont
		mov	hrs,#000H	;hrs = 0
inctime1:	ret			;return
;
;******************************************************************************
; routine settime 
;******************************************************************************
;This routine takes care of adjusting the time. after entering the routine the
;time is first set to 00:00:00. the hours digits are then incremented at the
;pace of the second input. When the key is pressed the counting of the hours
;stops, and the minutes start counting. The same procedure is followed for the
;minutes. After again pressing the kkey the clock continues at the time set.
;
;inputs:   hrs     I : hours byte of the internal time 
;          min     I : minutes byte of the internal time
;	   sec     I : seconds byte of the internal time
;	   sec_new B : set if new second is found
;	   key	   B : set if key is pressed
;outputs:  hrs     I : hours byte of the internal time 
;          min     I : minutes byte of the internal time
;	   sec     I : seconds byte of the internal time
;internal: 
;registers used: C,IE,A
;subroutines used: getsec, convert 
;
; hrs:min:sec = 00:00:00
; do until key=1
;    if sec_new=1 then
;       sec_new=0
;	hrs = hrs + 1
;	if hrs=24 then
;          hrs=0
;       endif
;	write to display (convert)
;    end if
; end do
; key = 0 
; do until key=1
;    if sec_new=1 then
;       sec_new=0
;	min = min + 1
;	if hrs=60 then
;          hrs=0
;       endif
;	write to display (convert)
;    end if
; end do
; key = 0 
; do until key=1
; end do
; key = 0
;
settime:	mov	sec,#000H	;reset hrs,min,sec
		mov	min,#000H	;
		mov	hrs,#000H	;
					;first the hours section
settime1:	jb	key,settime3	;continue as soon as a key is pressed
		lcall	getsec		;poll for new second
		jnb	sec_new,settime1;
		clr	sec_new		;
		clr	C		;if new second then hrs = hrs+1
		mov	A,hrs		;
		add	A,#001H		;
		da	A		;
		mov	hrs,A		;
		cjne	A,#024H,settime2;reset hours if needed and start new 
		mov	hrs,#000H	;loop
settime2:	mov	IE,#008H	;disable interupts
		lcall	convert		;write to displays
		mov	IE,#088H	;enable interupts
		sjmp	settime1	;
settime3:	clr	key		;depress key
					;next minutes section
settime4:	jb	key,settime6	;continue as soon as a key is pressed
		lcall	getsec		;poll for new second
		jnb	sec_new,settime4;
		clr	sec_new		;
		clr	C		;if new second then hrs = hrs+1
		mov	A,min		;
		add	A,#001H		;
		da	A		;
		mov	min,A		;
		cjne	A,#060H,settime5;reset hours if needed and start new 
		mov	min,#000H	;loop
settime5:	mov	IE,#008H	;disable interupts
		lcall	convert		;write to displays
		mov	IE,#088H	;enable interupts
		sjmp	settime4	;
settime6:	clr	key		;depress key
					;wait for key to continue in clock mode
settime7:	jnb	key,settime7	;
		clr	key
		ret
;
;******************************************************************************
; routine test_A 
;******************************************************************************
;This routine tests the nixie tubes by stepping through the digits at the same
;time every time the key is pressed in this way 00:00:00 -> 11:11:11 -> 
;22:22:22. The routine exits after 9:99:99
;
;inputs:   hrs     I : hours byte of the internal time 
;          min     I : minutes byte of the internal time
;	   sec     I : seconds byte of the internal time
;	   key	   B : set if key is pressed
;outputs:  hrs     I : hours byte of the internal time 
;          min     I : minutes byte of the internal time
;	   sec     I : seconds byte of the internal time
;internal: 
;registers used: C,IE,A
;subroutines used: convert 
;
test_A:		mov	sec,#000H	;reset hrs,min,sec
		mov	min,#000H	;
		mov	hrs,#000H	;
					;step through all digits by hand
test_A1:	mov	IE,#008H	;disable interupts
		lcall	convert		;write to displays
		mov	IE,#088H	;enable interupts
test_A2:	jnb	key,test_A2	;wait for key
		clr 	key		;
		clr	C		;sec = sec+11
		mov	A,sec		;
		add	A,#011H		;
		da	A		;
		mov	sec,A		;
		clr	C		;min = min+11
		mov	A,min		;
		add	A,#011H		;
		da	A		;
		mov	min,A		;
		clr	C		;hrs = hrs+11
		mov	A,hrs		;
		add	A,#011H		;
		da	A		;
		mov	hrs,A		;
		cjne	A,#010H,test_A1 ;test if we need to exit the routine
		mov	sec,#000H	;reset hrs,min,sec
		mov	min,#000H	;
		mov	hrs,#000H	;
		ret			;
;
;******************************************************************************
; routine test_B 
;******************************************************************************
;This routine makes the outputs of p0,p1,p2 alternately all 0 or 1 toggled
;by pressing the key, after 5 loops the routine is exited.
;The writing of the ones and zero's to the output ports iis done via the
;T1 service routine so that the debounced key is available. 
;
;inputs:   hrs     I : hours byte of the internal time 
;          min     I : minutes byte of the internal time
;	   sec     I : seconds byte of the internal time
;	   key	   B : set if key is pressed
;outputs:  hrs     I : hours byte of the internal time 
;          min     I : minutes byte of the internal time
;	   sec     I : seconds byte of the internal time
;internal: 
;registers used: C,IE,A
;subroutines used: convert 
;
test_B:		mov	cnt1,#005H	;5 loops
test_B1:	mov	p0_o,#000H	;the T1 service routine will write
		mov	p1_o,#000H	;these zero's to the output ports
		mov	p2_o,#000H	;
		mov	p0_e,#000H	;
		mov	p1_e,#000H	;
		mov	p2_e,#000H	;
test_B2:	jnb	key,test_B2	;wait for key
		clr	key
		mov	p0_o,#0FFH	;the T1 service routine will write
		mov	p1_o,#0FFH	;these one's to the output ports
		mov	p2_o,#0FFH	;
		mov	p0_e,#0FFH	;
		mov	p1_e,#0FFH	;
		mov	p2_e,#0FFH	;
test_B3:	jnb	key,test_B3	;wait for key
		clr	key		;  
		djnz	cnt1,test_B1	;finish loops
		ret
;
;******************************************************************************
; routine test_C 
;******************************************************************************
;This routine tests the nixie tubes by stepping through the digits at the same
;time every second in this way 00:00:00 -> 11:11:11 -> 22:22:22
;the routine is infinite and can only be exited by a hardware reset
;
;inputs:   hrs     I : hours byte of the internal time 
;          min     I : minutes byte of the internal time
;	   sec     I : seconds byte of the internal time
;	   key	   B : set if key is pressed
;outputs:  hrs     I : hours byte of the internal time 
;          min     I : minutes byte of the internal time
;	   sec     I : seconds byte of the internal time
;internal: 
;registers used: C,IE,A
;subroutines used: convert 
;
test_C:		mov	sec,#000H	;reset hrs,min,sec
		mov	min,#000H	;
		mov	hrs,#000H	;
					;step through all digits every second
test_C1:	mov	IE,#008H	;disable interupts
		lcall	convert		;write to displays
		mov	IE,#088H	;enable interupts
test_C2:	lcall	getsec		;
		jnb	sec_new,test_C2	;wait for key
		clr 	sec_new		;
		clr	C		;sec = sec+11
		mov	A,sec		;
		add	A,#011H		;
		da	A		;
		mov	sec,A		;
		clr	C		;min = min+11
		mov	A,min		;
		add	A,#011H		;
		da	A		;
		mov	min,A		;
		clr	C		;hrs = hrs+11
		mov	A,hrs		;
		add	A,#011H		;
		da	A		;
		mov	hrs,A		;
		sjmp	test_C1		;
		ret			;the return that is never executed
;
;******************************************************************************
; routine test_D 
;******************************************************************************
;This routine dissables the T1 interrupt service routine and "counts" the 
;output ports. p0 lowest order, p2 fastest order. exit only by hard reset.  
;
;inputs:    
;outputs:   
;internal: 
;registers used: p0,p1,p2
;subroutines used:
;
test_D:		mov	IE,#008H	;dissable interrupt
		mov	p0,#000H	;clear ports
		mov	p1,#001H	;
		mov	p2,#002H	;
test_D1:	djnz	p0,test_D1	;countdown ports
  		djnz	p1,test_D1	;
		djnz	p2,test_D1	;
		sjmp	test_D1
		ret
;
;******************************************************************************
; routine getmode 
;******************************************************************************
;This routine gets the clock mode. It uses only the most right digit from the 
;displays. The digit is incremented from 0 to 5 every second. When the key is
;pressed the routine exits with the current digit number in "mode"
;
;inputs:   sec_new IB: set if new second is found 
;	   key	   B : set if key is pressed
;outputs:  mode    I : clock mode 
;internal: 
;registers used: C,IE,A
;subroutines used: convertm 
;
; mode = 0
; write mode to display bytes (only morst right digit)
; do until key=1
;    if sec_new=1 then
;       sec_new=0
;       mode = mode+1
;	if mode=5 then
;          mode=0
;	endif
;    endif
; enddo
; key=0
; acc=mode	
;
getmode:	mov	mode,#000H	;reset mode
getmode1:	mov	IE,#008H	;disable interupts
		lcall	convertm	;write mode to lower nibble seconds
		mov	IE,#088H	;enable interupts
getmode2:	jb	key,getmode3	;exit if key pressed
		lcall	getsec		;poll for seconds
		jnb	sec_new,getmode2 ;if else wait for second
		clr	sec_new		;if second increment mode
		inc	mode		;	
		mov	A,mode		;
		cjne	A,#007H,getmode1;maximum modes
		sjmp	getmode		;
getmode3:	clr	key
		mov	A,mode
		ret
;
;******************************************************************************
; routine chk_blnk 
;******************************************************************************
;this routine checks (in a rather stupid way) if the tubes need to be blanked
;during the night. It sets the tbs_off flag if the blank request flag (blank)
;is set and the hrs is 0..5 
;
;inputs:   hrs     I : hours in BCD 
;	   blank   B : set if for blank mode
;outputs:  tbs_off B : flag for convert routine (set if blanking is needed) 
;internal: 
;registers used:
;subroutines used: 
; 
chk_blnk:	mov	A,hrs		;
		clr	tbs_off		;
		jnb	blank,chk_blnk7	;exit if blanking is not needed
		cjne	A,#000H,chk_blnk1 ; blank if is hours 0 
		sjmp	chk_blnk6
chk_blnk1:	cjne	A,#001H,chk_blnk2 ; blank if is hours 1 
		sjmp	chk_blnk6
chk_blnk2:	cjne	A,#002H,chk_blnk3 ; blank if is hours 2 
		sjmp	chk_blnk6
chk_blnk3:	cjne	A,#003H,chk_blnk4 ; blank if is hours 3 
		sjmp	chk_blnk6
chk_blnk4:	cjne	A,#004H,chk_blnk5 ; blank if is hours 4 
		sjmp	chk_blnk6
chk_blnk5:	cjne	A,#005H,chk_blnk7 ; blank if is hours 5 
chk_blnk6:	setb	tbs_off
chk_blnk7:	ret



		end		
